# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8 compliant>

import bpy

def particle_get_settings(context):
    if context.particle_system:
        return context.particle_system.settings
    elif isinstance(context.space_data.pin_id, bpy.types.ParticleSettings):
        return context.space_data.pin_id
    return None

# Use some of the existing buttons.
from bl_ui import properties_render
properties_render.RENDER_PT_render.COMPAT_ENGINES.add('POVRAY_RENDER')
properties_render.RENDER_PT_dimensions.COMPAT_ENGINES.add('POVRAY_RENDER')
properties_render.RENDER_PT_shading.COMPAT_ENGINES.add('POVRAY_RENDER')
properties_render.RENDER_PT_output.COMPAT_ENGINES.add('POVRAY_RENDER')
del properties_render

from bl_ui import properties_scene
for member in dir(properties_scene):
    subclass = getattr(properties_scene, member)
    try:
        subclass.COMPAT_ENGINES.add('POVRAY_RENDER')
    except:
        pass
del properties_scene

# Use only a subset of the world panels
from bl_ui import properties_world
properties_world.WORLD_PT_preview.COMPAT_ENGINES.add('POVRAY_RENDER')
properties_world.WORLD_PT_context_world.COMPAT_ENGINES.add('POVRAY_RENDER')
properties_world.WORLD_PT_world.COMPAT_ENGINES.add('POVRAY_RENDER')
#properties_world.WORLD_PT_mist.COMPAT_ENGINES.add('POVRAY_RENDER')
del properties_world



from bl_ui.properties_texture import context_tex_datablock

    
    
from bl_ui import properties_data_mesh
for member in dir(properties_data_mesh):
    subclass = getattr(properties_data_mesh, member)
    try:
        subclass.COMPAT_ENGINES.add('POVRAY_RENDER')
    except:
        pass
del properties_data_mesh

from bl_ui import properties_data_camera
del properties_data_camera.DATA_PT_camera
del properties_data_camera.DATA_PT_camera_display
del properties_data_camera.DATA_PT_camera_dof
#del properties_data_camera.DATA_PT_context_camera
del properties_data_camera.DATA_PT_custom_props_camera
del properties_data_camera.DATA_PT_lens

for member in dir(properties_data_camera):

    subclass = getattr(properties_data_camera, member)
    try:
        subclass.COMPAT_ENGINES.add('POVRAY_RENDER')
    except:
        pass
del properties_data_camera

from bl_ui import properties_data_lamp
del properties_data_lamp.DATA_PT_area
#del properties_data_lamp.DATA_PT_context_lamp
del properties_data_lamp.DATA_PT_custom_props_lamp
del properties_data_lamp.DATA_PT_falloff_curve
del properties_data_lamp.DATA_PT_lamp
#del properties_data_lamp.DATA_PT_preview
del properties_data_lamp.DATA_PT_shadow
del properties_data_lamp.DATA_PT_spot
del properties_data_lamp.DATA_PT_sunsky

for member in dir(properties_data_lamp):
    subclass = getattr(properties_data_lamp, member)
    try:
        subclass.COMPAT_ENGINES.add('POVRAY_RENDER')
    except:
        pass
del properties_data_lamp

from bl_ui import properties_particle as properties_particle
for member in dir(properties_particle):  # add all "particle" panels from blender
    subclass = getattr(properties_particle, member)
    try:
        subclass.COMPAT_ENGINES.add('POVRAY_RENDER')
    except:
        pass
del properties_particle

class RenderButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "render"

    @classmethod
    def poll(cls, context):
        rd = context.scene.render
        return (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)



class WorldButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "world"

    @classmethod
    def poll(cls, context):
        rd = context.scene.render
        world = context.world
        return (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)

class DataButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "data"

    @classmethod
    def poll(cls, context):
        rd = context.scene.render
        lamp = context.lamp
        return lamp and (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)
        
class MaterialButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "material"

    @classmethod
    def poll(cls, context):
        mat = context.material
        rd = context.scene.render
        return mat and (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)


class TextureButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "texture"

    @classmethod
    def poll(cls, context):
        tex = context.texture
        rd = context.scene.render
        return tex and (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)

class ObjectButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "object"

    @classmethod
    def poll(cls, context):
        obj = context.object
        rd = context.scene.render
        return obj and (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)

class CameraButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "data"

    @classmethod
    def poll(cls, context):
        cam = context.camera
        rd = context.scene.render
        return cam and (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)

class ParticleButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "particle"

    @classmethod
    def poll(cls, context):
        ob = context.object
        rd = context.scene.render
        return ob and (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)

class TextButtonsPanel():
    bl_space_type = 'TEXT_EDITOR'
    bl_region_type = 'UI'
    bl_label = "P.O.V-Ray"

    @classmethod
    def poll(cls, context):
        text = context.space_data
        rd = context.scene.render
        return text and (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)

class ToolsPanel(bpy.types.Panel):
    bl_label = "Povray hair"
    bl_space_type = "VIEW_3D"
    bl_region_type = "TOOLS"
 
    def draw(self, context):
        self.layout.operator("ob.exporthair",text="Include hair")

class Particle_PT_povray(ParticleButtonsPanel, bpy.types.Panel):
    bl_label = "Povray particle"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        part = context.particle_system
        rd = context.scene.render
        return part and (rd.use_game_engine is False) and (rd.engine in cls.COMPAT_ENGINES)

    def draw(self, context):
        part = particle_get_settings(context)
        layout=self.layout
        layout.prop_search(part.pov,"hair_material", context.blend_data, "materials")
        layout.prop(part.pov,"hair_type")
        if part.pov.hair_type == "SPHERESWEEP":
            layout.prop(part.pov,"sweep_tolerance")
        layout.label(text="Thickness")
        row=layout.row(align=True)
        row.prop(part.pov,"thickness_root")
        row.prop(part.pov,"thickness_tip")
##################LAMP################################################
class Data_PT_povray_lamp(DataButtonsPanel, bpy.types.Panel):
    bl_label = "Lamp"
    COMPAT_ENGINES = {'POVRAY_RENDER'}


    def draw(self, context):
        lamp = context.lamp
        layout = self.layout
        align=True
        layout.prop(lamp,"type", text="Type")
        layout.prop(lamp.pov, "media_interaction", text="Media interaction")
        layout.prop(lamp.pov, "media_attenuation", text="Media attenuation")
        row=layout.row(align=align)
        row.prop(lamp,"color", text="")
        row.prop(lamp,"energy", text="")
        if lamp.type not in {'SUN', 'HEMI'}:
            layout.prop(lamp,"distance", text="Distance")
        if lamp.type not in {'SUN', 'AREA', 'HEMI'}:
            layout.prop(lamp, "falloff_type", text="Falloff type")
        if lamp.type == "SPOT":
            layout.label(text="Spot:")
            col=layout.column(align=align)
            col.prop(lamp, "spot_size", text="Size")
            row=col.row()
            row.prop(lamp.pov, "falloff_spot", text="Falloff")
            row.prop(lamp, "spot_blend", text="Blend")
        if lamp.type == "AREA":
            layout.label(text="Area:")
            col=layout.column(align=align)
            row=col.row()
            row.prop(lamp, "size", text="Size x")
            row.prop(lamp, "size_y", text="Size y")
            row=col.row()
            row.prop(lamp, "shadow_ray_samples_x", text="Samples x")
            row.prop(lamp, "shadow_ray_samples_y", text="Samples y")

##################WORLD################################################
class World_PT_povray(WorldButtonsPanel, bpy.types.Panel):
    bl_label = "Povray Atmospheric Effects"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    def draw(self, context):
        world = context.world
        layout = self.layout
        align=True
        layout.prop(world.pov, "ambient_enable", text="Ambient color enable")
        layout.prop(world.pov, "fog_use", text="Use mist")
        if world.pov.fog_use:
            if len(world.povray_fogs) > 0:
                for layer in world.povray_fogs:
                    box=layout.box()
                    col=box.column(align=align)
                    col.label(text="Layer %s"%(int(layer.index)+1))
                    row=col.row(align=align)
                    row.prop(layer, "fog_type")
                    row.prop(layer, "fog_color", text="")
                    row=col.row(align=align)
                    row.prop(layer, "fog_distance")
                    row.prop(layer, "fog_offset")
                    col.prop(layer, "fog_alt")
                    row=col.row(align=align)
                    row.prop(layer, "fog_up_x")
                    row.prop(layer, "fog_up_y")
                    row.prop(layer, "fog_up_z")
                    col.label(text="Turbulence:")
                    row=col.row(align=align)
                    row.prop(layer, "fog_turb_x",text="X")
                    row.prop(layer, "fog_turb_y",text="Y")
                    row.prop(layer, "fog_turb_z",text="Z")
                    row=col.row(align=align)
                    row.prop(layer, "fog_turb_depth")
                    row.prop(layer, "fog_lambda")
                    row=col.row(align=align)
                    row.prop(layer, "fog_octaves")
                    row.prop(layer, "fog_omega")
            col=layout.column(align=align)
            col.operator("world.foglayernew")
            row=col.row(align=align)
            row.prop(world.pov, "fog_layer", text="Layer")
            row.operator("world.foglayerremove")
###################################################################
                
class RENDER_PT_povray_export_settings(RenderButtonsPanel, bpy.types.Panel):
    bl_label = "Export Settings"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    def draw(self, context):
        layout = self.layout

        scene = context.scene

        layout.active = (scene.pov.max_trace_level != 0)
        split = layout.split()

        col = split.column()
        col.label(text="Command line switches:")
        col.prop(scene.pov, "command_line_switches", text="")
        split = layout.split()
        split.prop(scene.pov, "tempfiles_enable", text="OS Tempfiles")
        if not scene.pov.tempfiles_enable:
            split.prop(scene.pov, "deletefiles_enable", text="Delete files")

        if not scene.pov.tempfiles_enable:
            col = layout.column()
            col.prop(scene.pov, "scene_name", text="Name")
            col.prop(scene.pov, "scene_path", text="Path to files")
            #col.prop(scene.pov, "scene_path", text="Path to POV-file")
            #col.prop(scene.pov, "renderimage_path", text="Path to image")

            split = layout.split()
            split.prop(scene.pov, "indentation_character", text="Indent")
            if scene.pov.indentation_character == 'SPACE':
                split.prop(scene.pov, "indentation_spaces", text="Spaces")

            row = layout.row()
            row.prop(scene.pov, "comments_enable", text="Comments")
            row.prop(scene.pov, "list_lf_enable", text="Line breaks in lists")

class RENDER_PT_povray_antialias(RenderButtonsPanel, bpy.types.Panel):
    bl_label = "Anti-Aliasing"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    def draw_header(self, context):
        scene = context.scene
        self.layout.prop(scene.pov, "antialias_enable", text="")
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        layout.active = scene.pov.antialias_enable
        row = layout.row()
        row.prop(scene.pov, "antialias_method", text="")
        row.prop(scene.pov, "jitter_enable", text="Jitter")
        split = layout.split()
        col = split.column()
        col.prop(scene.pov, "antialias_depth", text="AA Depth")
        sub = split.column()
        sub.prop(scene.pov, "jitter_amount", text="Jitter Amount")
        if scene.pov.jitter_enable:
            sub.enabled = True
        else:
            sub.enabled = False
        row = layout.row()
        row.prop(scene.pov, "antialias_threshold", text="AA Threshold")
        row.prop(scene.pov, "antialias_gamma", text="AA Gamma")

class RENDER_PT_povray_render_settings(RenderButtonsPanel, bpy.types.Panel):
    bl_label = "Global Settings"
    COMPAT_ENGINES = {'POVRAY_RENDER'}
    def draw(self, context):
        scene = context.scene
        layout=self.layout
        row=layout.row(align=True)
        row.prop(scene.pov, "assumed_gamma")
        row.prop(scene.pov, "display_gamma")
        layout.prop(scene.pov, "max_trace_level", text="Ray Depth")
        layout.prop(scene.pov, "mm_per_unit")
        layout.prop(scene.pov, "sss_enable")
        if scene.pov.sss_enable:
            row=layout.row()
            row.prop(scene.pov, "sss_samples_enable")
            row.prop(scene.pov, "sss_radiosity")
            if scene.pov.sss_samples_enable:
                layout.label(text="Subsurface samples:")
                row=layout.row(align=True)
                row.prop(scene.pov, "sss_samples_max", text="Max")
                row.prop(scene.pov, "sss_samples_min", text="Min")

class RENDER_PT_povray_global_photons(RenderButtonsPanel, bpy.types.Panel):
    bl_label = "Global photons"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    def draw_header(self, context):
        scene = context.scene
        self.layout.prop(scene.pov, "photon_enable", text="")

    def draw(self, context):
        scene = context.scene
        if scene.pov.photon_enable:
            layout = self.layout
            align=True
            row=layout.row(align=align)
            row.prop(scene.pov, "photon_count", text="Count")
            row.prop(scene.pov, "photon_max_trace_level")
            layout.prop(scene.pov, "photon_media_enable", text="Photon media enable")
            if scene.pov.photon_media_enable:
                row=layout.row(align=align)
                row.prop(scene.pov, "photon_media_steps", text="Max steps")
                row.prop(scene.pov, "photon_media_factor", text="Factor")
            layout.prop(scene.pov, "photon_advanced")
            if scene.pov.photon_advanced:
                col = layout.column(align=align)
                row=col.row()
                row.prop(scene.pov, "photon_adc")
                if scene.pov.photon_adc:
                    row.prop(scene.pov, "photon_adc_bailout")
                col.prop(scene.pov,"photon_gather")
                if scene.pov.photon_gather:
                    row=col.row()
                    row.prop(scene.pov, "photon_gather_min")
                    row.prop(scene.pov, "photon_gather_max")
                row=col.row()
                row.prop(scene.pov,"photon_autostop")
                if scene.pov.photon_autostop:
                    row.prop(scene.pov,"photon_autostop_value")
            layout.label(text="Photon map file:")
            row=layout.row()
            if not scene.pov.photon_openfile:
                row.prop(scene.pov, "photon_savefile")
            if not scene.pov.photon_savefile:
                row.prop(scene.pov, "photon_openfile")

class RENDER_PT_povray_radiosity(RenderButtonsPanel, bpy.types.Panel):
    bl_label = "Radiosity"
    COMPAT_ENGINES = {'POVRAY_RENDER'}
    def draw_header(self, context):
        scene = context.scene
        self.layout.prop(scene.pov, "radio_enable", text="")
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        layout.active = scene.pov.radio_enable
        split = layout.split()
        col = split.column()
        col.prop(scene.pov, "radio_count", text="Rays")
        col.prop(scene.pov, "radio_recursion_limit", text="Recursions")
        split.prop(scene.pov, "radio_error_bound", text="Error Bound")
        layout.prop(scene.pov, "radio_display_advanced")
        if scene.pov.radio_display_advanced:
            split = layout.split()
            col = split.column()
            col.prop(scene.pov, "radio_adc_bailout", slider=True)
            col.prop(scene.pov, "radio_gray_threshold", slider=True)
            col.prop(scene.pov, "radio_low_error_factor", slider=True)
            col.prop(scene.pov, "radio_pretrace_start", slider=True)
            col = split.column()
            col.prop(scene.pov, "radio_brightness")
            col.prop(scene.pov, "radio_minimum_reuse", text="Min Reuse")
            col.prop(scene.pov, "radio_nearest_count")
            col.prop(scene.pov, "radio_pretrace_end", slider=True)
            split = layout.split()
            col = split.column()
            col.label(text="Estimation Influence:")
            col.prop(scene.pov, "radio_media")
            col.prop(scene.pov, "radio_normal")
            split.prop(scene.pov, "radio_always_sample")
class RENDER_PT_povray_media(RenderButtonsPanel, bpy.types.Panel):
    bl_label = "Atmosphere Media"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    def draw_header(self, context):
        scene = context.scene

        self.layout.prop(scene.pov, "media_enable", text="")

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        layout.active = scene.pov.media_enable
        row = layout.row()
        row.prop(scene.pov, "media_samples", text="Samples")
        row.prop(scene.pov, "media_color", text="")

class MATERIAL_PT_povray_materials(MaterialButtonsPanel, bpy.types.Panel):    ######################### ADD povray parameters
    bl_label = "Materials"
    COMPAT_ENGINES = {'POVRAY_RENDER'}
    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        return (context.material or context.object) and (engine in cls.COMPAT_ENGINES)
    def draw(self, context):
        layout = self.layout
        mat = context.material
        ob = context.object
        slot = context.material_slot
        space = context.space_data
        if ob:
            row = layout.row()
            row.template_list("MATERIAL_UL_matslots", "", ob, "material_slots", ob, "active_material_index", rows=1)
            col = row.column(align=True)
            col.operator("object.material_slot_add", icon='ZOOMIN', text="")
            col.operator("object.material_slot_remove", icon='ZOOMOUT', text="")
        if ob:
            layout.template_ID(ob, "active_material", new="material.new")
        if mat:
            if ob.mode == 'EDIT':
                row = layout.row(align=True)
                row.operator("object.material_slot_assign", text="Assign")
                row.operator("object.material_slot_select", text="Select")
                row.operator("object.material_slot_deselect", text="Deselect")
            layout.prop(mat, "type", expand=True)
            layout.prop(mat.pov,"material_use_nodes")
            layout.label(text="Preview:")
            layout.template_preview(context.material)

class MATERIAL_PT_povray_material_texture(MaterialButtonsPanel, bpy.types.Panel):######################### ADD povray parameters
    bl_label = "Material"
    COMPAT_ENGINES = {'POVRAY_RENDER'}
    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        ob=context.object
        mat=context.material
        return (mat and (engine in cls.COMPAT_ENGINES))

    def draw(self, context):
        scene=context.scene
        mat=context.material
        layout = self.layout
        align=True
        layout.prop(mat.pov, "povray_mat_type")
        if mat.pov.povray_mat_type == 'DEFAULT':
            layout.prop(mat.pov, "povray_finish",text="Finish")
            if mat.pov.povray_finish:
                ob=context.object
                scn=context.scene
                align=True
                layout.prop(mat.pov, "finish_as_mat",text="Use material")
                layout.label(text="__________________________")
                row=layout.row()
                row.prop(mat.pov, "m_use_ambient")
                if mat.pov.m_use_ambient:
                    row.prop(mat.pov, "m_ambient_color",text="")
                layout.label(text="__________________________")
                row=layout.row()
                row.prop(mat.pov, "m_use_diffuse")
                if mat.pov.m_use_diffuse:
                    row.prop(mat.pov, "m_diffuse_albedo")
                    row=layout.row(align=align)
                    if mat.pov.finish_as_mat:
                        row.prop(mat,"diffuse_color",text="")
                    row.prop(mat, "diffuse_intensity", text="Intensity")
                    if mat.pov.finish_as_mat:
                        layout.prop(mat.pov,"m_transparent")
                        if mat.pov.m_transparent:
                            row=layout.row(align=align)
                            row.prop(mat.pov, "m_transmit")
                            row.prop(mat.pov, "m_filter")
                layout.label(text="__________________________")
                row=layout.row()
                row.prop(mat.pov, "m_use_emission")
                if mat.pov.m_use_emission:
                    row.prop(mat, "emit", text="")
                layout.label(text="__________________________")
                row=layout.row()
                row.prop(mat.pov, "m_use_brilliance")
                if mat.pov.m_use_brilliance:
                    row.prop(mat.pov, "m_brilliance", text="")
                layout.label(text="__________________________")
                row=layout.row()
                row.prop(mat.pov, "m_use_spec")
                if mat.pov.m_use_spec:
                    row.prop(mat.pov, "spec_use_albedo", text="Albedo")
                    row.prop(mat.pov, "spec_use_phong", text="Phong")
                    row = layout.row(align=align)
                    row.prop(mat, "specular_intensity", text="Intensity")
                    row.prop(mat.pov, "spec_size")
                    row = layout.row()
                    row.prop(mat.pov, "m_use_specmetallic")
                    if mat.pov.m_use_specmetallic:
                        row.prop(mat.pov, "m_specmetallic",text="")
                layout.label(text="__________________________")
                row=layout.row()
                row.prop(mat.pov, "m_use_crand")
                if mat.pov.m_use_crand:
                    row.prop(mat.pov, "m_crand",text='')
                layout.label(text="__________________________")
                layout.prop(mat.pov, "conserve_energy", text="Conserve energy")
                layout.label(text="__________________________")
                layout.prop(mat.raytrace_mirror, "use", text="Mirror")
                if mat.raytrace_mirror.use:
                    row = layout.row()
                    row.prop(mat, "mirror_color", text="")
                    row.prop(mat.pov, "m_reflect_fresnel", text="Fresnel")
                    col=layout.column(align=align)
                    row = col.row(align=align)
                    row.prop(mat.raytrace_mirror, "reflect_factor", text="Reflect")
                    row.prop(mat.pov, "m_metallic", text="Metallic",slider=True)
                    row = col.row(align=align)
                    row.prop(mat.raytrace_mirror, "fresnel", text="Falloff",slider=True)
                    row.prop(mat.raytrace_mirror, "fresnel_factor", text="Exponent")
                if scn.pov.sss_enable:
                    layout.label(text="__________________________")
                    row = layout.row(align=align)
                    row.prop(mat.pov, "sss_mat_enable", text="Translucence")
                    if mat.pov.sss_mat_enable:
                        row.prop(mat.pov, "sss_translucency_color",text="")
                layout.label(text="__________________________")
                layout.prop(mat.pov, "irid_enable", text="Iridescence")
                if mat.pov.irid_enable:
                    row = layout.row()
                    row.label(text="Amount")
                    row.label(text="Thick")
                    row.label(text="Turb")
                    row = layout.row(align=align)
                    row.prop(mat.pov, "irid_amount", text="", slider=True)
                    row.prop(mat.pov, "irid_thickness", text="", slider=True)
                    row.prop(mat.pov, "irid_turbulence", text="", slider=True)
        if mat.pov.povray_mat_type == 'LAYERED':
            if len(mat.povray_layered_texture)>0:
                row=layout.row()
                row.label(text="N")
                row.label(text="Pigment")
                row.label(text="Normal")
                row.label(text="Finish")
            number=-1
            for block in mat.povray_layered_texture:
                col=layout.column(align=align)
                if block:
                    number+=1
                    row=col.row(align=align)
                    row.label(text="%s"%number)
                    row.prop_search(block, "layer_pigment",scene,"povray_pigments",text="")
                    row.prop_search(block, "layer_normal",scene,"povray_normals",text="")
                    row.prop_search(block, "layer_finish",scene,"povray_finishes",text="")
            col=layout.column(align=align)
            col.operator("mat.layernew")
            row=col.row(align=align)
            row.prop(mat.pov,"m_layer_number")
            row.operator("mat.layerremove",icon="X")
        if mat.pov.povray_mat_type == 'PIGMENTPATTERN':
            layout.prop_search(mat.pov, "m_pp",scene,"povray_pigment_patterns")
            col=layout.column(align=align)
            row=col.row(align=align)
            row.prop(mat.pov,"m_first_value",text="")
            row.prop_search(mat.pov,"m_pp_first",context.blend_data,"materials",text="")
            row=col.row(align=align)
            row.prop(mat.pov,"m_second_value",text="")
            row.prop_search(mat.pov,"m_pp_second",context.blend_data,"materials",text="")

        if mat.pov.povray_mat_type == 'PATTERN':
            layout.label(text="The texture map is necessary!")
            layout.label(text="Go to the panel of textures!")


class MATERIAL_PT_povray_media(MaterialButtonsPanel, bpy.types.Panel):
    bl_label = "Media"
    COMPAT_ENGINES = {'POVRAY_RENDER'}
    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        mat=context.material
        return (mat and mat.type == "VOLUME" and (engine in cls.COMPAT_ENGINES))

    def draw(self, context):
        layout = self.layout
        mat = context.material
        align = True
        row=layout.row()
        row.prop(mat.pov, "absorption_media_ob", text="Absorption")
        row.prop(mat.pov, "absorption_val", text="Value")
        row=layout.row()
        row.prop(mat.pov, "emission_media_ob", text="Emission")
        row.prop(mat.pov, "emission_val", text="Value")
        layout.prop(mat.pov, "scattering_media_ob", text="Scattering")
        if mat.pov.scattering_media_ob:
            row = layout.row()
            row.label(text="Scattering color:")
            row.prop(mat.pov, "s_media_color", text="")
            col=layout.column(align=align)
            row = col.row()
            row.prop(mat.pov, "s_media_type", text="Type")
            row.prop(mat.pov, "s_samples", text="Samples")
            row = col.row()
            row.prop(mat.pov, "ext_s_media", text="Extinction")
            if mat.pov.s_media_type == 5:
                row.prop(mat.pov, "ecc_s_media", text="Eccentric")


class TEXTURE_PT_povray_texslot(TextureButtonsPanel, bpy.types.Panel):
    bl_label = "Textures"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        #if not hasattr(context, "texture_slot"):
            #return False

        return (engine in cls.COMPAT_ENGINES)

    def draw(self, context):
        space = context.space_data
        mat = context.material
        world=context.world
        slot = getattr(context, "texture_slot", None)
        idblock = context_tex_datablock(context)
        pin_id = space.pin_id
        layout = self.layout
        layout.prop(space, "texture_context", expand=True)       
        if mat or context.world:
            row = layout.row()
            row.template_list("TEXTURE_UL_texslots", "", idblock, "texture_slots", idblock, "active_texture_index", rows=2)        
            col = row.column(align=True)
            col.operator("texture.slot_move", text="", icon='TRIA_UP').type = 'UP'
            col.operator("texture.slot_move", text="", icon='TRIA_DOWN').type = 'DOWN'
            #col.menu("TEXTURE_MT_specials", icon='DOWNARROW_HLT', text="")        
            col = layout.column(align=True)
            col.template_ID(idblock, "active_texture", new="texture.new")
     
class TEXTURE_PT_povray_preview(TextureButtonsPanel, bpy.types.Panel):
    bl_label = "Preview"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        if not hasattr(context, "texture_slot"):
            return False
        tex=context.texture
        mat=context.material
        return (tex and (engine in cls.COMPAT_ENGINES))

    def draw(self, context):
        tex = context.texture
        slot = getattr(context, "texture_slot", None)
        idblock = context_tex_datablock(context)
        layout = self.layout
        if idblock:
            layout.template_preview(tex, parent=idblock, slot=slot)
        else:
            layout.template_preview(tex, slot=slot)
        if tex.pov.tex_type == 'PATTERN':
            layout.operator("tex.preview_update")

class TEXTURE_PT_povray_active_texture(TextureButtonsPanel, bpy.types.Panel):
    bl_label = "Active texture"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        if not hasattr(context, "texture_slot"):
            return False
        tex=context.texture
        return (tex and (engine in cls.COMPAT_ENGINES))

    def draw(self, context):
        tex = context.texture
        mat=context.material
        scene=context.scene
        ob=context.object
        align=True
        layout=self.layout
        if mat:
            layout.label(text="Assign as:")
            if mat.type=="SURFACE":
                if not tex.pov.tex_as_normal and not tex.pov.tex_as_ppattern and not tex.pov.tex_as_material and mat.pov.povray_mat_type!="PATTERN":
                    layout.prop(tex.pov, "tex_as_pigment")
                if not tex.pov.tex_as_pigment and not tex.pov.tex_as_ppattern and not tex.pov.tex_as_material and mat.pov.povray_mat_type!="PATTERN":
                    row=layout.row()
                    row.prop(tex.pov, "tex_as_normal")
                    if tex.pov.tex_as_normal and tex.pov.nor_type == "IMAGE":
                        row.prop(tex.pov, "tex_nor", text="Bump")
                if not tex.pov.tex_as_pigment and not tex.pov.tex_as_normal and not tex.pov.tex_as_material and mat.pov.povray_mat_type!="PATTERN":
                    layout.prop(tex.pov, "tex_as_ppattern")
                if not tex.pov.tex_as_pigment and not tex.pov.tex_as_normal and not tex.pov.tex_as_ppattern and mat.pov.povray_mat_type=="PATTERN":
                    layout.prop(tex.pov, "tex_as_material",text="Texture map")
                if tex.pov.tex_as_pigment  or tex.pov.tex_as_normal or tex.pov.tex_as_ppattern or tex.pov.tex_as_material:
                    if tex.pov.tex_type!='COLOR' or tex.pov.nor_type!='GRAY':
                        layout.prop(tex.pov, "tex_coords", text="Coords")
                    if tex.pov.tex_as_pigment or tex.pov.tex_as_ppattern or tex.pov.tex_as_material:
                        layout.prop(tex.pov, "tex_type", text="Type")
                    if tex.pov.tex_as_normal:
                        layout.prop(tex.pov, "nor_type", text="Type")
                    if tex.pov.tex_as_pigment and tex.pov.tex_type=='COLOR':
                        layout.prop(tex.pov, "tex_pigment_color")
                        row=layout.row(align=True)
                        row.prop(tex.pov, "tex_pigment_filter")
                        row.prop(tex.pov, "tex_pigment_transmit")
                    if tex.pov.tex_as_normal and tex.pov.nor_type=='GRAY':
                        layout.prop(tex.pov, "tex_normal_gray")
                    if tex.pov.tex_type == "IMAGE" or tex.pov.nor_type == "IMAGE":
                        layout.prop(tex.pov, "cr_create_tex")
                        if tex.pov.cr_create_tex:
                            #layout.prop(scene.pov,"pycairo_pyversion")
                            row=layout.row(align=align)
                            row.prop(ob.pov, "cr_image_width")
                            row.prop(ob.pov, "cr_image_height")
                            layout.label(text=tex.pov.tex_image_path)
                        else:
                            layout.prop_search(tex.pov, "tex_images_data", context.blend_data, "images")
                            layout.label(text="Open texture image:")
                        layout.prop(tex.pov, "tex_image_path",text="")
                        row=layout.row(align=True)
                        row.prop(tex.pov, "tex_pigment_filter")
                        row.prop(tex.pov, "tex_pigment_transmit")
                        if tex.pov.tex_as_material:
                            row=layout.row()
                            row.prop(tex.pov, "tex_ip_alpha")
                            layout.prop(tex.pov, "tex_ip_premul")
                            if not tex.pov.tex_ip_alpha:
                                row.prop(tex.pov, "tex_ip_gamma")
                        space = context.space_data
                        if tex.pov.cr_create_tex:
                            row=layout.row(align=align) 
                            row.prop(ob.pov,"cr_image_bg")
                            if ob.pov.cr_image_bg=="COLOR":
                                row.prop(ob.pov,"cr_image_bg_color",text="")
                            layout.operator("tex.cairo_create")
                            layout.operator("export_png_format.png")
                            layout.label(text=tex.pov.tex_image_path)
                    if tex.pov.tex_type == "PATTERN" or tex.pov.nor_type == "PATTERN":
                        layout.prop(tex.pov, "colored_patterns", text="Pattern")
                        if tex.pov.tex_as_material:
                            layout.label(text="Texture map:") 
                            if len(tex.povray_texture) > 0:
                                row=layout.row(align=True)
                                split = row.split(percentage=0.1)
                                split.label(text="N")
                                row=split.row(align=True)
                                row.label(text="Value")
                                row.label(text="Material")
                            row = layout.row()
                            split = row.split(percentage=0.1)
                            col = split.column()
                            number=-1
                            for entry in tex.povray_texture:
                                number+=1
                                row=col.row(align=True)
                                row.label(text="%s:"%number)
                            col = split.column(align=True)
                            for entry in tex.povray_texture:
                                row=col.row(align=True)
                                row.prop(entry, "texture_value",text="")
                                row.prop_search(entry, "name",context.blend_data, "materials",text="",icon='MATERIAL')
                            col=layout.column(align=True)
                            col.operator("tex.textureentrynew")
                            row=col.row(align=True)
                            row.prop(tex.pov,"pigment_entry_number")
                            row.operator("tex.textureentryremove",icon='X')
                        if tex.pov.tex_as_pigment or tex.pov.tex_as_ppattern:
                            layout.template_color_ramp(tex, "color_ramp", expand=True)
                            layout.prop(tex.pov, "tex_use_pmap")
                            if tex.pov.tex_use_pmap:
                                col=layout.column(align=True)
                                if len(tex.povray_pigment) > 0:
                                    row=col.row(align=True)
                                    row.label(text="Color stop")
                                    row.label(text="Pigment")
                                    row.label(text="Transmit")
                                number=-1
                                for entry in tex.povray_pigment:
                                    number+=1
                                    row=col.row(align=True)
                                    row.prop(entry, "pigment_number",text="%s   "%number)
                                    row.prop_search(entry, "pigment_name",context.scene, "povray_pigments",text="",icon='TEXTURE')
                                    row.prop(entry, "color_transmit",text="",slider=True)
                                col=layout.column(align=True)
                                col.operator("tex.pigmententrynew")
                                row=col.row(align=True)
                                row.prop(tex.pov,"pigment_entry_number")
                                row.operator("tex.pigmententryremove",icon='X')
                        if tex.pov.tex_as_normal:
                            if not tex.pov.tex_nor_color_map:
                                row=layout.row()
                                row.prop(tex.pov, "tex_use_slope")
                                if tex.pov.tex_use_slope:
                                    row.prop(tex.pov,"tex_slope_accuracy",slider=True)
                            if not tex.pov.tex_use_slope:
                                layout.prop(tex.pov, "tex_nor",slider=True)
                                layout.prop(tex.pov, "tex_nor_color_map")
                            if tex.pov.tex_use_slope:
                                col=layout.column(align=True)
                                if len(tex.povray_slope) > 0:
                                    row=col.row(align=True)
                                    row.label(text="N")
                                    row.label(text="Value")
                                    row.label(text="Height")
                                    row.label(text="Slope")
                                number=-1
                                for entry in tex.povray_slope:
                                    number+=1
                                    row=col.row(align=True)
                                    row.label(text="%s"%number)
                                    row.prop(entry, "slope_value",text="")
                                    row.prop(entry, "slope_height",text="", slider=True)
                                    row.prop(entry, "slope_slope",text="")
                                col=layout.column(align=True)
                                col.operator("tex.slopeentrynew")
                                row=col.row(align=True)
                                row.prop(tex.pov,"slope_entry_number")
                                row.operator("tex.slopeentryremove",icon='X')
                            if tex.pov.tex_nor_color_map:
                                layout.template_color_ramp(tex, "color_ramp", expand=True)
            if mat.type=="VOLUME":
                if not tex.pov.tex_use_emis:
                    row=layout.row()
                    row.prop(tex.pov, "tex_use_absorp", text="")
                    row.prop(tex.pov, "tex_absorp", text="Absorption")
                if not tex.pov.tex_use_absorp:
                    row=layout.row()
                    row.prop(tex.pov, "tex_use_emis", text="")
                    row.prop(tex.pov, "tex_emis", text="Emission")
                if tex.pov.tex_use_emis or tex.pov.tex_use_absorp:
                    layout.prop(tex.pov, "tex_coords", text="Coords")
                    layout.prop(tex.pov, "tex_type", text="Type")
                    if tex.pov.tex_type=='PATTERN':
                        layout.prop(tex.pov, "colored_patterns", text="Pattern")
                        layout.template_color_ramp(tex, "color_ramp", expand=True) 
        if context.world:
            layout.prop(tex.pov, "tex_image_path")

class TEXTURE_PT_povray_transform(TextureButtonsPanel, bpy.types.Panel):
    bl_label = "Transformations"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        tex=context.texture
        if not hasattr(context, "texture_slot") :
            return False
        return (tex and engine in cls.COMPAT_ENGINES and
               (((tex.pov.tex_as_pigment or tex.pov.tex_as_ppattern  or tex.pov.tex_as_material) and tex.pov.tex_type in {'PATTERN','IMAGE'}) or
                (tex.pov.tex_as_normal and tex.pov.nor_type in {'PATTERN','IMAGE'})))

    def draw(self, context):
        tex = context.texture
        mat = context.material
        layout = self.layout
        align=True
        if tex:
            if tex.pov.tex_type == "PATTERN" or tex.pov.nor_type == "PATTERN" or tex.pov.tex_use_slope:
            ###################################################Patterns################################
                if tex.pov.colored_patterns == 'agate':
                    layout.prop(tex.pov, "agate_turbulence", text="Agate Turbulence",slider=True)
                if tex.pov.colored_patterns == 'crackle':
                    layout.label(text="Crackle form vector")
                    row=layout.row(align=align)
                    row.prop(tex.pov, "crackle_form_x", text="X")
                    row.prop(tex.pov, "crackle_form_y", text="Y")
                    row.prop(tex.pov, "crackle_form_z", text="Z")
                    layout.prop(tex.pov, "crackle_metric", text="Crackle metric")
                    layout.prop(tex.pov, "crackle_solid", text="Crackle solid")
                if tex.pov.colored_patterns in {'spiral1', 'spiral2'}:
                    layout.prop(tex.pov, "modifier_numbers", text="Number of arms")
                if tex.pov.colored_patterns == 'tiling':
                    layout.prop(tex.pov, "modifier_numbers", text="Pattern number")
                if tex.pov.colored_patterns == 'magnet':
                    layout.prop(tex.pov, "magnet_style", text="Magnet style")
                if tex.pov.colored_patterns == 'quilted':
                    row = layout.row(align=align)
                    row.prop(tex.pov, "modifier_control0", text="Control0")
                    row.prop(tex.pov, "modifier_control1", text="Control1")
                if tex.pov.colored_patterns == 'brick':
                    col = layout.column(align=align)
                    row = col.row()
                    row.prop(tex.pov, "brick_size_x", text="Brick size X")
                    row.prop(tex.pov, "brick_size_y", text="Brick size Y")
                    row=col.row()
                    row.prop(tex.pov, "brick_size_z", text="Brick size Z")
                    row.prop(tex.pov, "brick_mortar", text="Brick mortar")
                if tex.pov.colored_patterns in {'julia','mandel','magnet'}:
                    col = layout.column(align=align)
                    if tex.pov.colored_patterns == 'julia':
                        row = col.row()
                        row.prop(tex.pov, "julia_complex_1", text="Complex 1")
                        row.prop(tex.pov, "julia_complex_2", text="Complex 2")
                    if tex.pov.colored_patterns == 'magnet' and tex.pov.magnet_style == 'julia':
                        row = col.row()
                        row.prop(tex.pov, "julia_complex_1", text="Complex 1")
                        row.prop(tex.pov, "julia_complex_2", text="Complex 2")
                    row=col.row()
                    if tex.pov.colored_patterns in {'julia','mandel'}:            
                        row.prop(tex.pov, "f_exponent", text="Exponent")
                    if tex.pov.colored_patterns == 'magnet':            
                        row.prop(tex.pov, "magnet_type", text="Type")
                    row.prop(tex.pov, "f_iter", text="Iterations")
                    row=col.row()
                    row.prop(tex.pov, "f_ior", text="Interior")
                    row.prop(tex.pov, "f_ior_fac", text="Factor I")
                    row=col.row()
                    row.prop(tex.pov, "f_eor", text="Exterior")
                    row.prop(tex.pov, "f_eor_fac", text="Factor E")
                if tex.pov.colored_patterns == 'gradient':        
                    layout.label(text="Gradient orientation:")
                    column_flow = layout.column_flow(columns=3, align=True)
                    column_flow.prop(tex.pov, "grad_orient_x", text="X")        
                    column_flow.prop(tex.pov, "grad_orient_y", text="Y")
                    column_flow.prop(tex.pov, "grad_orient_z", text="Z")
                if tex.pov.colored_patterns == 'pavement':
                    layout.prop(tex.pov, "pave_sides", text="Pavement:number of sides")
                    col = layout.column(align=align)
                    column_flow = col.column_flow(columns=3, align=True)
                    column_flow.prop(tex.pov, "pave_tiles", text="Tiles")        
                    if tex.pov.pave_sides == '4' and tex.pov.pave_tiles == 6:
                        column_flow.prop(tex.pov, "pave_pat_35", text="Pattern")
                    if tex.pov.pave_sides == '6' and tex.pov.pave_tiles == 5:
                        column_flow.prop(tex.pov, "pave_pat_22", text="Pattern")
                    if tex.pov.pave_sides == '4' and tex.pov.pave_tiles == 5:
                        column_flow.prop(tex.pov, "pave_pat_12", text="Pattern")
                    if tex.pov.pave_sides == '3' and tex.pov.pave_tiles == 6:
                        column_flow.prop(tex.pov, "pave_pat_12", text="Pattern")
                    if tex.pov.pave_sides == '6' and tex.pov.pave_tiles == 4:
                        column_flow.prop(tex.pov, "pave_pat_7", text="Pattern")
                    if tex.pov.pave_sides == '4' and tex.pov.pave_tiles == 4:
                        column_flow.prop(tex.pov, "pave_pat_5", text="Pattern")
                    if tex.pov.pave_sides == '3' and tex.pov.pave_tiles == 5:
                        column_flow.prop(tex.pov, "pave_pat_4", text="Pattern")
                    if tex.pov.pave_sides == '6' and tex.pov.pave_tiles == 3:
                        column_flow.prop(tex.pov, "pave_pat_3", text="Pattern")
                    if tex.pov.pave_sides == '3' and tex.pov.pave_tiles == 4:
                        column_flow.prop(tex.pov, "pave_pat_3", text="Pattern")
                    if tex.pov.pave_sides == '4' and tex.pov.pave_tiles == 3:
                        column_flow.prop(tex.pov, "pave_pat_2", text="Pattern")
                    if tex.pov.pave_sides == '6' and tex.pov.pave_tiles == 6:
                        column_flow.label(text="!!! 5 tiles!")
                    column_flow.prop(tex.pov, "pave_form", text="Form")
                if tex.pov.colored_patterns == 'function':
                    layout.prop(tex.pov, "func_list", text="Functions") 
                if tex.pov.colored_patterns == 'function' and tex.pov.func_list != "NONE":
                    func = None
                    if tex.pov.func_list in {"f_noise3d", "f_ph", "f_r", "f_th"}:
                        func = 0
                    if tex.pov.func_list in {"f_comma","f_crossed_trough","f_cubic_saddle",
                                             "f_cushion","f_devils_curve","f_enneper","f_glob",
                                             "f_heart","f_hex_x","f_hex_y","f_hunt_surface",
                                             "f_klein_bottle","f_kummer_surface_v1",
                                             "f_lemniscate_of_gerono","f_mitre","f_nodal_cubic",
                                             "f_noise_generator","f_odd","f_paraboloid","f_pillow",
                                             "f_piriform","f_quantum","f_quartic_paraboloid",
                                             "f_quartic_saddle","f_sphere","f_steiners_roman",
                                             "f_torus_gumdrop","f_umbrella"}:
                        func = 1
                    if tex.pov.func_list in {"f_bicorn","f_bifolia","f_boy_surface","f_superellipsoid","f_torus"}:
                        func = 2
                    if tex.pov.func_list in {"f_ellipsoid","f_folium_surface","f_hyperbolic_torus",
                                             "f_kampyle_of_eudoxus","f_parabolic_torus",
                                             "f_quartic_cylinder","f_torus2"}:
                        func = 3
                    if tex.pov.func_list in {"f_blob2","f_cross_ellipsoids","f_flange_cover",
                                             "f_isect_ellipsoids","f_kummer_surface_v2","f_ovals_of_cassini",
                                             "f_rounded_box","f_spikes_2d","f_strophoid"}:
                        func = 4
                    if tex.pov.func_list in {"f_algbr_cyl1","f_algbr_cyl2","f_algbr_cyl3","f_algbr_cyl4",
                                             "f_blob","f_mesh1","f_poly4","f_spikes"}:
                        func = 5
                    if tex.pov.func_list in {"f_devils_curve_2d","f_dupin_cyclid","f_folium_surface_2d",
                                             "f_hetero_mf","f_kampyle_of_eudoxus_2d","f_lemniscate_of_gerono_2d",
                                             "f_polytubes","f_ridge","f_ridged_mf","f_spiral","f_witch_of_agnesi"}:
                        func = 6
                    if tex.pov.func_list in {"f_helix1","f_helix2","f_piriform_2d","f_strophoid_2d"}:
                        func = 7
                    if tex.pov.func_list == "f_helical_torus":
                        func = 8
                    column_flow = layout.column_flow(columns=3, align=True)
                    column_flow.label(text="X")        
                    column_flow.prop(tex.pov, "func_plus_x", text="")
                    column_flow.prop(tex.pov, "func_x", text="Value")
                    column_flow = layout.column_flow(columns=3, align=True)
                    column_flow.label(text="Y")        
                    column_flow.prop(tex.pov, "func_plus_y", text="")
                    column_flow.prop(tex.pov, "func_y", text="Value")
                    column_flow = layout.column_flow(columns=3, align=True)
                    column_flow.label(text="Z")        
                    column_flow.prop(tex.pov, "func_plus_z", text="")
                    column_flow.prop(tex.pov, "func_z", text="Value")
                    row=layout.row(align=align)
                    if func > 0:
                        row.prop(tex.pov, "func_P0", text="P0")
                    if func > 1:
                        row.prop(tex.pov, "func_P1", text="P1")
                    row=layout.row(align=align)
                    if func > 2:
                        row.prop(tex.pov, "func_P2", text="P2")
                    if func > 3:
                        row.prop(tex.pov, "func_P3", text="P3")
                    row=layout.row(align=align)
                    if func > 4:
                        row.prop(tex.pov, "func_P4", text="P4")
                    if func > 5:
                        row.prop(tex.pov, "func_P5", text="P5")
                    row=layout.row(align=align)
                    if func > 6:
                        row.prop(tex.pov, "func_P6", text="P6")
                    if func > 7:
                        row.prop(tex.pov, "func_P7", text="P7")
                        row=layout.row(align=align)
                        row.prop(tex.pov, "func_P8", text="P8")
                        row.prop(tex.pov, "func_P9", text="P9")
            ###################################################End Patterns############################


            layout.prop(tex.pov, "warp_types", text="Warp types") #warp
            if tex.pov.warp_types == "REPEAT":
                row=layout.row()
                row.prop(tex.pov, "warp_repeat_x")
                if tex.pov.warp_repeat_x:
                    row.prop(tex.pov, "rx_value")
                    row=layout.row()
                    row.prop(tex.pov, "rx_offset")
                    row.prop(tex.pov, "rx_offset_value")
                    row=layout.row()
                    row.label(text="Flip:")
                    row.prop(tex.pov, "rx_flip_x")
                    row.prop(tex.pov, "rx_flip_y")
                    row.prop(tex.pov, "rx_flip_z")
                row=layout.row()
                row.prop(tex.pov, "warp_repeat_y")
                if tex.pov.warp_repeat_y:
                    row.prop(tex.pov, "ry_value")
                    row=layout.row()
                    row.prop(tex.pov, "ry_offset")
                    row.prop(tex.pov, "ry_offset_value")
                    row=layout.row()
                    row.label(text="Flip:")
                    row.prop(tex.pov, "ry_flip_x")
                    row.prop(tex.pov, "ry_flip_y")
                    row.prop(tex.pov, "ry_flip_z")
                row=layout.row()
                row.prop(tex.pov, "warp_repeat_z")
                if tex.pov.warp_repeat_z:
                    row.prop(tex.pov, "rz_value")
                    row=layout.row()
                    row.prop(tex.pov, "rz_offset")
                    row.prop(tex.pov, "rz_offset_value")
                    row=layout.row()
                    row.label(text="Flip:")
                    row.prop(tex.pov, "rz_flip_x")
                    row.prop(tex.pov, "rz_flip_y")
                    row.prop(tex.pov, "rz_flip_z")
            if tex.pov.warp_types == "TOROIDAL":
                layout.prop(tex.pov, "warp_tor_major_radius", text="Major radius",slider=True)
            if tex.pov.warp_types not in {"CUBIC","NONE"}:
                layout.prop(tex.pov, "warp_orientation", text="Warp orientation",slider=True)
            col=layout.column(align=align)
            col.prop(tex.pov, "warp_dist_exp", text="Distance exponent",slider=True)        
            row = col.row(align=align)
            row.prop(tex.pov, "modifier_frequency", text="Frequency",slider=True)
            row.prop(tex.pov, "modifier_phase", text="Phase",slider=True)

            row=layout.row()

            row.label(text="Offset:")
            row.label(text="Scale:")
            row.label(text="Rotate:")
            col=layout.column(align=align) 
            row=col.row(align=align)
            row.prop(tex.pov, "tex_mov_x", text="X",slider=True)
            row.prop(tex.pov, "tex_scale_x", text="X",slider=True)
            row.prop(tex.pov, "tex_rot_x", text="X",slider=True)
            row=col.row(align=align)
            row.prop(tex.pov, "tex_mov_y", text="Y",slider=True)
            row.prop(tex.pov, "tex_scale_y", text="Y",slider=True)
            row.prop(tex.pov, "tex_rot_y", text="Y",slider=True)
            row=col.row(align=align)
            row.prop(tex.pov, "tex_mov_z", text="Z",slider=True)
            row.prop(tex.pov, "tex_scale_z", text="Z",slider=True)
            row.prop(tex.pov, "tex_rot_z", text="Z",slider=True)
            row=layout.row()

            row.label(text="Turbulence:")
            col=layout.column(align=align) 
            row=col.row(align=align)
            row.prop(tex.pov, "warp_turbulence_x", text="X",slider=True)
            row.prop(tex.pov, "modifier_octaves", text="Octaves",slider=True)
            row=col.row(align=align)
            row.prop(tex.pov, "warp_turbulence_y", text="Y",slider=True)
            row.prop(tex.pov, "modifier_lambda", text="Lambda",slider=True)
            row=col.row(align=align)
            row.prop(tex.pov, "warp_turbulence_z", text="Z",slider=True)
            row.prop(tex.pov, "modifier_omega", text="Omega",slider=True)

class OBJECT_PT_povray_write_object(ObjectButtonsPanel, bpy.types.Panel):    ######################### ADD povray parameters
    bl_label = "Write"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        return (context.object and context.object.type in {"MESH","CURVE"} and (engine in cls.COMPAT_ENGINES))

    def draw(self, context):
        obj = context.object
        scene=context.scene
        align=True
        layout = self.layout
        layout.prop(obj.pov, "object_write_type")
        if obj.pov.object_write_type == 'BOOLEAN':
            layout.prop(obj.pov, "boolean_parent")
            if obj.pov.boolean_parent==False:
                split=layout.split(percentage=0.9, align=True)
                row=split.row()
                row.prop_search(obj.pov,"boolean_data", context.scene, "povray_boolean_objects",icon='RADIOBUT_ON')
                row=split.row()
                row.operator("ob.update_pbo",text="",icon='FILE_REFRESH')
        if obj.pov.object_write_type not in {'CAIRO','SMB','BOOLEAN'}:
            row=layout.row()
            row.prop(obj.pov, "povobject_visible")
            if obj.pov.object_write_type == 'SPHERESWEEP':
                row.prop(obj.pov,"shape_ss_type")
            if obj.pov.object_write_type == 'LATHE':
                row.prop(obj.pov,"lathe_spline_type")
        if obj.pov.object_write_type == 'SUPERELLIPSOID':
            row=layout.row(align=align)
            row.prop(obj.pov, "shape_se_param1", text="")
            row.prop(obj.pov, "shape_se_param2", text="")
        if obj.pov.object_write_type == 'SUPERTORUS':
            row=layout.row(align=align)
            row.prop(obj.pov, "shape_st_radius_major")
            row.prop(obj.pov, "shape_st_radius_minor")
            row=layout.row(align=align)
            row.prop(obj.pov, "shape_st_ring")
            row.prop(obj.pov, "shape_st_cross")
        if obj.pov.object_write_type == 'ROUNDPYRAMID':
            row=layout.row()
            row.label(text="Border:")
            row.prop(obj.pov, "shape_rp_border")
            row=layout.row(align=align)
            row.prop(obj.pov, "shape_rp_filled")
            row.prop(obj.pov, "shape_rp_merge")
        if obj.pov.object_write_type == 'CAIRO':
            layout.prop(obj.pov, "cr_fill")
            row=layout.row(align=align)
            row.prop(obj.pov, "cr_fill_color",text="")
            row.prop(obj.pov, "cr_fill_color_alpha",text="alpha",slider=True)
            row=layout.row(align=align)
            row.prop(obj.pov, "cr_stroke")
            if obj.pov.cr_stroke:
                row.prop(obj.pov, "cr_line_width")
                row=layout.row(align=align)
                row.prop(obj.pov, "cr_stroke_color",text="")
                row.prop(obj.pov, "cr_stroke_color_alpha",text="alpha",slider=True)
                row=layout.row(align=align)
                row.prop(obj.pov, "cr_cap_style")
                row.prop(obj.pov, "cr_join_style")
            layout.prop(obj.pov, "cr_composition")
        if obj.pov.object_write_type == 'SMB':
            align=align
            row = layout.row()
            row.label(text="Scattering color:")
            row.prop(obj.pov, "smb_color", text="")
            col=layout.column(align=align)
            row = col.row()
            row.prop(obj.pov, "smb_type", text="Type")
            row.prop(obj.pov, "smb_samples", text="Samples")
            row = col.row()
            row.prop(obj.pov, "smb_ext", text="Extinction")
            if obj.pov.smb_type == 5:
                row.prop(obj.pov, "smb_ecc", text="Eccentric")
        if obj.pov.object_write_type == 'HEIGHTFIELD':
            layout.prop(obj.pov, "shape_hf_image")
            row = layout.row(align=align)
            row.prop(obj.pov, "shape_hf_gamma")
            row.prop(obj.pov, "shape_hf_water")
            row = layout.row()
            row.prop(obj.pov, "shape_hf_premultiplied")
            row.prop(obj.pov, "shape_hf_smooth")
            layout.prop(obj.pov, "shape_hf_hierarchy")

class OBJECT_PT_povray_copy_object(ObjectButtonsPanel, bpy.types.Panel):    ######################### ADD povray parameters
    bl_label = "Copy"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        return (context.object and context.object.type in {"MESH","CURVE"} and (engine in cls.COMPAT_ENGINES))

    def draw(self, context):
        obj = context.object
        layout = self.layout
        row = layout.row()
        row.prop(obj.pov, "copy_around")
        if obj.pov.copy_around:
            row.prop(obj.pov, "around_axis")
        row=layout.row(align=True)
        row.prop(obj.pov, "number_copy")
        if obj.pov.copy_around:
            row.prop(obj.pov, "copy_distance",text="Radius")
        else:
            row.prop(obj.pov, "copy_distance",text="Distance")
        layout.operator("ob.objectcopy")

class OBJECT_PT_povray_interior(ObjectButtonsPanel, bpy.types.Panel):    ######################### ADD povray parameters
    bl_label = "Interior, Photons and Radiosity"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        return (context.object and context.object.type in {"MESH","CURVE"} and
                context.object.pov.object_write_type not in {'CAIRO','SMB'} and 
                (engine in cls.COMPAT_ENGINES))

    def draw(self, context):
        obj = context.object
        scene = context.scene
        slots=obj.material_slots
        matVolume = 0
        for slot in slots:        
            if slot and slot.material and slot.material.type == "VOLUME":
                matVolume += 1
        align = True
        layout = self.layout
        if matVolume == 0:        
            layout.prop(obj.pov, "m_ior", text="IOR")
            row = layout.row()
            row.prop(obj.pov, "refraction_caustics", text="Refractive caustics")
            if obj.pov.refraction_caustics:
                if scene.pov.photon_enable:
                    row.prop(obj.pov, "refraction_photons")
                if (obj.pov.refraction_photons==False or scene.pov.photon_enable==False):
                    layout.prop(obj.pov, "fake_caustics_power")
                if obj.pov.refraction_photons and scene.pov.photon_enable:
                    row=layout.row(align=align)
                    row.prop(obj.pov, "photons_dispersion")
                    row.prop(obj.pov, "photons_dispersion_samples")
                layout.label(text="Fade:")
                row=layout.row(align=align)
                row.prop(obj.pov, "fade_distance", text="Distance")
                row.prop(obj.pov, "fade_power", text="Power")
                row=layout.row()
                row.label(text="Fade color")
                row.prop(obj.pov, "fade_color", text="")
            if scene.pov.photon_enable:
                layout.prop(obj.pov, "reflection_photons")
                layout.prop(obj.pov, "collect_photons", text="Collect Photon Caustics")
                layout.prop(obj.pov, "target_photons", text="Target") 
                layout.prop(obj.pov, "pass_through") 
        if scene.pov.radio_enable:
            col = layout.column()
            col.label(text="Radiosity:")
            col.prop(obj.pov, "importance_value", text="Importance")

class OBJECT_PT_povray_ob_modifiers(ObjectButtonsPanel, bpy.types.Panel):
    bl_label = "Other object modifiers"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        return (context.object and context.object.type in {"MESH","CURVE"} and
                context.object.pov.object_write_type not in {'CAIRO','SMB'} and 
                (engine in cls.COMPAT_ENGINES))

    def draw(self, context):
        obj = context.object
        if obj.type in {"MESH","CURVE"}:
            layout = self.layout
            layout.prop(obj.pov, "noshadow")
            layout.prop(obj.pov, "noreflection")
            layout.prop(obj.pov, "noimage")

class OBJECT_PT_povray_boolean(ObjectButtonsPanel, bpy.types.Panel):
    bl_label = "POV-Ray Boolean"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    @classmethod
    def poll(cls, context):
        engine = context.scene.render.engine
        return (context.object and context.object.type in {"MESH","CURVE","META"} and
                context.object.pov.object_write_type not in {'CAIRO','SMB'} and 
                (engine in cls.COMPAT_ENGINES))

    def draw_header(self, context):
        obj = context.object
        self.layout.prop(obj.pov, "povray_boolean", text="")

    def draw(self, context):
        obj = context.object
        if obj.pov.povray_boolean:
            layout = self.layout
            col = layout.column()
            col.label(text="Select first operation:")        
            col.prop(obj.pov, "boolean_operation_1", text="")
            col.prop_search(obj.pov, "boolean_object_1", context.scene, "objects")
            col.prop_search(obj.pov, "boolean_group_1", context.blend_data, "groups")
            col.label(text="Select second operation:")  
            col.prop(obj.pov, "boolean_operation_2", text="")
            col.prop_search(obj.pov, "boolean_object_2", context.scene, "objects")
            col.prop_search(obj.pov, "boolean_group_2", context.blend_data, "groups")
            col.label(text="Select third operation")  
            col.prop(obj.pov, "boolean_operation_3", text="")
            col.prop_search(obj.pov, "boolean_object_3", context.scene, "objects")
            col.prop_search(obj.pov, "boolean_group_3", context.blend_data, "groups")
            col.operator("ob.booleanmesh")

class CAMERA_PT_povray_cam_set(CameraButtonsPanel, bpy.types.Panel):
    bl_label = "Settings"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    def draw(self, context):
        layout = self.layout
        cam = context.camera
        layout.prop(cam.pov, "cam_type")
        if cam.pov.cam_type == "perspective":
            layout.prop(cam.pov, "cam_architectural")
        if cam.pov.cam_type == 'cylindrical':
            layout.prop(cam.pov, "cylinder_type", text="Cylinder type")
        if cam.pov.cam_type == 'mesh_camera':
            row=layout.row(align=True)
            row.prop(cam.pov, "rays_per_pixel", text="Number")
            row.prop(cam.pov, "mesh_cam_distr", text="Distr")
            layout.prop_search(cam.pov, "mesh_cam_obj", context.scene, "objects")
        layout.prop(cam, "lens", text="Lens")
        layout.label(text="Manual correction of aspect ratio:")
        layout.prop(cam.pov, "correction_aspect_ratio")

class CAMERA_PT_povray_cam_dof(CameraButtonsPanel, bpy.types.Panel):
    bl_label = "DOF"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    def draw_header(self, context):
        cam = context.camera

        self.layout.prop(cam.pov, "dof_enable", text="")

    def draw(self, context):
        layout = self.layout

        cam = context.camera

        layout.active = cam.pov.dof_enable
        layout.prop_search(cam, "dof_object", context.scene, "objects")
        layout.prop(cam.pov, "dof_aperture")

        split = layout.split()

        col = split.column()
        col.prop(cam.pov, "dof_samples_min")
        col.prop(cam.pov, "dof_variance")

        col = split.column()
        col.prop(cam.pov, "dof_samples_max")
        col.prop(cam.pov, "dof_confidence")





class TEXT_PT_povray_custom_code(TextButtonsPanel, bpy.types.Panel):
    bl_label = "P.O.V-Ray"
    COMPAT_ENGINES = {'POVRAY_RENDER'}

    def draw(self, context):
        layout = self.layout

        text = context.space_data.text
        if text:
            layout.prop(text.pov, "custom_code", text="Add as POV code")

#TEXTURE_PT_povray_colors.COMPAT_ENGINES.add('POVRAY_RENDER')
